param (
    $Username, 
    $Password
    )

BEGIN
{
    Import-Module MSOnline -Force

	$SecurePassword = ConvertTo-SecureString $Password -AsPlainText -Force
	$Creds = New-Object System.Management.Automation.PSCredential $Username, $SecurePassword

	Connect-MsolService -Credential $Creds

    #Some default License values
    $DefaultUsageLocation = "SE"

    #Staff licenses
    $STANDARDWOFFPACK_FACULTY = "domain:STANDARDWOFFPACK_FACULTY"
    $PROJECTONLINE_PLAN_1_FACULTY = "domain:PROJECTONLINE_PLAN_1_FACULTY"

    #Student license
    $STANDARDWOFFPACK_STUDENT = "domain:STANDARDWOFFPACK_STUDENT"
    $PROJECTONLINE_PLAN_1_STUDENT = "domain:PROJECTONLINE_PLAN_1_STUDENT"
    
    #Alumni licenses
    $EXCHANGESTANDARD_ALUMNI = "domain:EXCHANGESTANDARD_ALUMNI"
    
    #Some parts should be excluded otherwise Project and Standard licenses will conflict
    $StndDisabledPlans= @()
    $StndDisabledPlans +="SHAREPOINTSTANDARD_EDU"
    $StndDisabledPlans +="SHAREPOINTWAC_EDU"

    $FacultyStnd = New-MsolLicenseOptions -AccountSkuId $STANDARDWOFFPACK_FACULTY -DisabledPlans $StndDisabledPlans;
    $StudStnd = New-MsolLicenseOptions -AccountSkuId $STANDARDWOFFPACK_STUDENT -DisabledPlans $StndDisabledPlans;
}

PROCESS
{
#Initialize Parameters
$Identifier = $_.Identifier
$objectGuid = $_.DN
$ErrorName = "success"
$ErrorDetail = $null
$date = Get-Date -Format "yyyy-MM-dd"

#Verify changetype.
if ($_.ObjectModificationType -eq 'Add')
	{
    # adds are caught by importing new objects from Azure AD (see import script)
	# and joining these to existing user objects on the metaverse
	throw "Add modification are not supported"
	}
	
if ($_.ObjectModificationType -eq 'Delete')
	{
    # deletes are caught by importing deleted objects from Azure AD 
	throw "Delete modification are not supported"
	}

#Supported ChangeType is Replace
if ($_.ObjectModificationType -match 'Replace')
{
#Reset values
$IsLicensed = $null
$LicenseType = $null

#Get the user
$msoluser=Get-MsolUser -UserPrincipalName $_.DN
$IsLicensed=$msoluser.IsLicensed

#Loop through changes and update parameters
foreach ($can in $_.ChangedAttributeNames)
    {# $can : ChangedAttributeName
    foreach ($ValueChange in $_.AttributeChanges[$can].ValueChanges)
        {
        if ( $can -eq 'IsLicensed' ){$IsLicensed = $ValueChange.Value}
        if ( $can -eq 'LicenseType' ){$LicenseType = $ValueChange.Value}
        }
    }

#If NOT licensed Remove ALL Licenses from User
if(!$IsLicensed)
    {Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -RemoveLicenses $msoluser.Licenses.AccountSkuId}

#LicenseType only relevant if IsLicensed is TRUE
if($IsLicensed)
    {
    #Set UsageLocation if it is missing
    if(!$msoluser.UsageLocation)
        {Set-MsolUser -UserPrincipalName $msolUser.UserPrincipalName -UsageLocation $DefaultUsageLocation;}

    #Remove any Previous Licenses
    Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -RemoveLicenses $msoluser.Licenses.AccountSkuId
    #Add the correct license
    if($LicenseType -eq "Staff")
        {
        Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -AddLicenses $STANDARDWOFFPACK_FACULTY -LicenseOptions $FacultyStnd;
        Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -AddLicenses $PROJECTONLINE_PLAN_1_FACULTY;
        }
    elseif($LicenseType -eq "Student")
        {
        Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -AddLicenses $STANDARDWOFFPACK_STUDENT -LicenseOptions $StudStnd;
        Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -AddLicenses $PROJECTONLINE_PLAN_1_STUDENT;
        }
    elseif($LicenseType -eq "Alumni")
        {
        Set-MsolUserLicense -UserPrincipalName $msolUser.UserPrincipalName -AddLicenses $EXCHANGESTANDARD_ALUMNI;
        }
    else
        {
        throw "Unknown LicenseType"
        }
    }
}
#Return the result to the MA
$obj = @{}
$obj.Add("[Identifier]",$Identifier)
$obj.Add("[ErrorName]",$ErrorName)
if($ErrorDetail){$obj.Add("[ErrorDetail]",$ErrorDetail)}
$obj
}

END
{
#To bad Connect-MsolService does not have a corresponding Disconnect so that we can do a nice cleanup!
}
